"""Output functionality."""

from __future__ import annotations

import decimal
import json
import logging
import shutil
import sys
import textwrap

from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import TYPE_CHECKING, TypeVar


if TYPE_CHECKING:
    from .utils import TermFeatures


T = TypeVar("T", bound="Level")
GOLDEN_RATIO = 1.61803398875


def round_half_up(number: float) -> int:
    """Round a number to the nearest integer with ties going away from zero.

    This is different the round() where exact halfway cases are rounded to the nearest
    even result instead of away from zero. (e.g. round(2.5) = 2, round(3.5) = 4).

    This will always round based on distance from zero. (e.g round(2.5) = 3, round(3.5) = 4).

    :param number: The number to round
    :returns: The rounded number as an it
    """
    rounded = decimal.Decimal(number).quantize(
        decimal.Decimal("1"),
        rounding=decimal.ROUND_HALF_UP,
    )
    return int(rounded)


def console_width() -> int:
    """Get a console width based on common screen widths.

    :returns: The console width
    """
    medium = 80
    wide = 132
    width = shutil.get_terminal_size().columns
    if width <= medium:
        return width
    if width <= wide:
        return max(80, round_half_up(width / GOLDEN_RATIO))
    return wide


class Color:
    """Color constants."""

    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    GREY = "\033[90m"  # Bright black?
    BRIGHT_RED = "\033[91m"
    BRIGHT_GREEN = "\033[92m"
    BRIGHT_YELLOW = "\033[93m"
    BRIGHT_BLUE = "\033[94m"
    BRIGHT_MAGENTA = "\033[95m"
    BRIGHT_CYAN = "\033[96m"
    BRIGHT_WHITE = "\033[97m"
    END = "\033[0m"


class Level(Enum):
    """An exit message prefix."""

    CRITICAL = "Critical"
    DEBUG = "Debug"
    ERROR = "Error"
    HINT = "Hint"
    INFO = "Info"
    NOTE = "Note"
    WARNING = "Warning"

    @property
    def log_level(self: Level) -> int:
        """Return a log level.

        :returns: The log level
        """
        mapping = {
            Level.CRITICAL: logging.CRITICAL,
            Level.DEBUG: logging.DEBUG,
            Level.ERROR: logging.ERROR,
            Level.HINT: logging.INFO,
            Level.INFO: logging.INFO,
            Level.NOTE: logging.INFO,
            Level.WARNING: logging.WARNING,
        }
        return mapping[self]

    @classmethod
    def _longest_name(cls: type[T]) -> int:
        """Return the longest exit message prefix.

        :returns: The longest exit message prefix
        """
        return max(len(member.value) for member in cls)

    @classmethod
    def longest_formatted(cls: type[T]) -> int:
        """Return the longest exit message prefix.

        :returns: The longest exit message prefix
        """
        return max(len(str(member)) for member in cls)

    def __str__(self: Level) -> str:
        """Return the exit message prefix as a string.

        :returns: The exit message prefix as a string
        """
        return (
            f"{' ' * (self._longest_name() - len(self.name))}{self.name.capitalize()}: "
        )


@dataclass
class Msg:
    """An object to hold a message to present when exiting."""

    #: The message that will be presented
    message: str
    #: The prefix for the message, used for formatting
    prefix: Level = Level.ERROR

    @property
    def color(self: Msg) -> str:
        """Return a color for the prefix.

        :returns: The color for the prefix
        """
        color_mapping = {
            Level.CRITICAL: Color.BRIGHT_RED,
            Level.DEBUG: Color.GREY,
            Level.ERROR: Color.RED,
            Level.HINT: Color.CYAN,
            Level.INFO: Color.MAGENTA,
            Level.NOTE: Color.GREEN,
            Level.WARNING: Color.YELLOW,
        }
        return color_mapping[self.prefix]

    def to_lines(
        self: Msg,
        color: bool,  # noqa: FBT001
        width: int,
        with_prefix: bool,  # noqa: FBT001
    ) -> list[str]:
        """Output exit message to the console.

        :param color: Whether to color the message
        :param width: Constrain message to width
        :param with_prefix: Whether to prefix the message
        :returns: The exit message as a string
        """
        prefix_length = Level.longest_formatted()
        indent = " " * prefix_length

        lines = []
        message_lines = self.message.splitlines()

        lines.extend(
            textwrap.fill(
                message_lines[0],
                width=width,
                break_on_hyphens=False,
                initial_indent=str(self.prefix) if with_prefix else indent,
                subsequent_indent=indent,
            ).splitlines(),
        )

        if len(message_lines) > 1:
            for line in message_lines[1:]:
                lines.extend(
                    textwrap.fill(
                        line,
                        width=width,
                        break_on_hyphens=False,
                        initial_indent=indent,
                        subsequent_indent=indent,
                    ).splitlines(),
                )

        start_color = self.color if color else ""
        end_color = Color.END if color else ""

        return [f"{start_color}{line}{end_color}" for line in lines]


class Output:
    """Output functionality."""

    def __init__(  # noqa: PLR0913
        self: Output,
        log_file: str,
        log_level: str,
        log_append: str,
        term_features: TermFeatures,
        verbosity: int,
        display: str = "text",
    ) -> None:
        """Initialize the output object.

        Args:
            log_file: The path to the log file
            log_level: The log level
            log_append: Whether to append to the log file
            term_features: Terminal features
            verbosity: The verbosity level
            display: Whether to output as text or JSON
        """
        self._verbosity = verbosity
        self.call_count: dict[str, int] = {
            "critical": 0,
            "debug": 0,
            "error": 0,
            "hint": 0,
            "info": 0,
            "note": 0,
            "warning": 0,
        }
        self.term_features = term_features
        self.logger = logging.getLogger("pip4a")
        if log_level != "notset":
            self.logger.setLevel(log_level.upper())
            self.log_to_file = bool(log_file)
            log_file_path = Path(log_file)
            if log_file_path.exists() and log_append == "false":
                log_file_path.unlink()
            formatter = logging.Formatter(
                fmt="%(asctime)s %(levelname)s '%(name)s.%(module)s.%(funcName)s' %(message)s",
            )
            handler = logging.FileHandler(log_file)
            handler.setFormatter(formatter)
            handler.setLevel(log_level.upper())
            self.logger.addHandler(handler)
            self.log_to_file = True
        else:
            self.log_to_file = False
        self.display = display

    def critical(self: Output, msg: str) -> None:
        """Print a critical message to the console.

        :param msg: The message to print
        """
        self.call_count["critical"] += 1
        self.log(msg, level=Level.CRITICAL)
        sys.exit(1)

    def debug(self: Output, msg: str) -> None:
        """Print a debug message to the console.

        :param msg: The message to print
        """
        self.call_count["debug"] += 1
        self.log(msg, level=Level.DEBUG)

    def error(self: Output, msg: str) -> None:
        """Print an error message to the console.

        :param msg: The message to print
        """
        self.call_count["error"] += 1
        self.log(msg, level=Level.ERROR)

    def hint(self: Output, msg: str) -> None:
        """Print a hint message to the console.

        :param msg: The message to print
        """
        self.call_count["hint"] += 1
        self.log(msg, level=Level.HINT)

    def info(self: Output, msg: str) -> None:
        """Print a info message to the console.

        :param msg: The message to print
        """
        self.call_count["info"] += 1
        self.log(msg, level=Level.INFO)

    def note(self: Output, msg: str) -> None:
        """Print a note message to the console.

        :param msg: The message to print
        """
        self.call_count["note"] += 1
        self.log(msg, level=Level.NOTE)

    def warning(self: Output, msg: str) -> None:
        """Print a warning message to the console.

        :param msg: The message to print
        """
        self.call_count["warning"] += 1
        self.log(msg, level=Level.WARNING)

    def log(self: Output, msg: str, level: Level = Level.ERROR) -> None:
        """Print a message to the console.

        :param msg: The message to print
        :param prefix: The prefix for the message
        """
        if self.log_to_file:
            self.logger.log(level.log_level, msg, stacklevel=3)

        debug = 2
        info = 1
        if (self._verbosity < debug and level == Level.DEBUG) or (
            self._verbosity < info and level == Level.INFO
        ):
            return

        if self.display == "json":
            print(  # noqa: T201
                json.dumps({"level": level.name, "msg": msg}),
                flush=True,
            )
            return

        lines = Msg(message=msg, prefix=level).to_lines(
            color=self.term_features.color,
            width=console_width(),
            with_prefix=True,
        )
        final_msg = "\n".join(lines)

        file = sys.stderr if level in [Level.CRITICAL, Level.ERROR] else sys.stdout

        print(final_msg, file=file)



"""Re-usable utility functions used by this package."""

from __future__ import annotations

import os
import sys

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ansible_creator.exceptions import CreatorError


if TYPE_CHECKING:
    from importlib import abc

    from ansible_creator.output import Output
    from ansible_creator.templar import Templar

PATH_REPLACERS = {
    "network_os": "collection_name",
    "resource": "resource",
}

if sys.version_info < (3, 10):
    import importlib_resources as resources
else:
    from importlib import resources


@dataclass
class TermFeatures:
    """Terminal features."""

    color: bool
    links: bool

    def any_enabled(self: TermFeatures) -> bool:
        """Return True if any features are enabled."""
        return any((self.color, self.links))


def get_file_contents(directory: str, filename: str) -> str:
    """Return contents of a file.

    :param directory: A directory within ansible_creator package.
    :param filename: Name of the file to read contents from.

    :returns: Content loaded from file as string.

    :raises FileNotFoundError: if filename cannot be located
    :raises TypeError: if invalid type is found
    :raises ModuleNotFoundError: if incorrect package is provided
    """
    package: str = f"ansible_creator.{directory}"

    try:
        with resources.files(package).joinpath(filename).open(
            "r",
            encoding="utf-8",
        ) as file_open:
            content: str = file_open.read()
    except (FileNotFoundError, TypeError, ModuleNotFoundError) as exc:
        msg = "Unable to fetch file contents.\n"
        raise CreatorError(msg) from exc

    return content


def expand_path(path: str) -> str:
    """Resolve absolute path.

    :param path: Path to expand.
    :returns: Expanded absolute path.
    """
    return os.path.abspath(
        os.path.expanduser(os.path.expandvars(path)),
    )


# TO-DO: move this to a better location, possible base class for all subcommands?
def copy_container(  # noqa: PLR0913
    source: str,
    dest: str,
    output: Output,
    templar: Templar,
    template_data: dict[str, str],
    allow_overwrite: list[str] | None = None,
) -> None:
    """Copy files and directories from a possibly nested source to a destination.

    :param source: Name of the source container.
    :param dest: Absolute destination path.
    :param templar: An object of template class.
    :param template_data: A dictionary containing data to render templates with.
    :param allow_overwrite: A list of paths that should be overwritten at destination.

    :raises CreatorError: if allow_overwrite is not a list.
    """
    output.debug(msg=f"starting recursive copy with source container '{source}'")
    output.debug(msg=f"allow_overwrite set to {allow_overwrite}")

    def _recursive_copy(root: abc.Traversable) -> None:
        """Recursively traverses a resource container and copies content to destination.

        :param root: A traversable object representing root of the container to copy.
        """
        output.debug(msg=f"current root set to {root}")

        for obj in root.iterdir():
            overwrite = False
            dest_name = str(obj).split(source + "/", maxsplit=1)[-1]
            dest_path = os.path.join(dest, dest_name)
            if (allow_overwrite) and (dest_name in allow_overwrite):
                overwrite = True
            # replace placeholders in destination path with real values
            for key, val in PATH_REPLACERS.items():
                if key in dest_path and template_data:
                    dest_path = dest_path.replace(key, template_data.get(val, ""))

            if obj.is_dir():
                if not os.path.exists(dest_path):
                    os.makedirs(dest_path)

                # recursively copy the directory
                _recursive_copy(root=obj)

            elif obj.is_file():
                # remove .j2 suffix at destination
                dest_file = os.path.join(dest, dest_path.split(".j2", maxsplit=1)[0])
                output.debug(msg=f"dest file is {dest_file}")

                # write at destination only if missing or belongs to overwrite list
                if not os.path.exists(dest_file) or overwrite:
                    content = obj.read_text(encoding="utf-8")
                    # only render as templates if both of these are provided
                    # templating is not mandatory
                    if templar and template_data:
                        content = templar.render_from_content(
                            template=content,
                            data=template_data,
                        )
                    with open(dest_file, "w", encoding="utf-8") as df_handle:
                        df_handle.write(content)

    _recursive_copy(root=resources.files(f"ansible_creator.resources.{source}"))



"""A schema validation helper for ansible-creator."""

from __future__ import annotations

from json import JSONDecodeError, loads


try:
    from jsonschema import SchemaError
    from jsonschema.validators import validator_for

    HAS_JSONSCHEMA = True
except ImportError:
    HAS_JSONSCHEMA = False

from ansible_creator.exceptions import CreatorError
from ansible_creator.utils import get_file_contents


class SchemaValidator:
    """Class representing a validation engine for ansible-creator content definition files."""

    def __init__(self: SchemaValidator, data: dict, criteria: str) -> None:
        """Initialize the validation engine.

        :param data: Content definition as a dictionary
        :param criteria: Name of the file that contains the JSON schema.
        """
        self.data = data
        self.criteria = criteria

    def validate(self: SchemaValidator) -> list:
        """Validate data against loaded schema.

        :returns: A list of schema validation errors (if any).
        :raises CreatorError: if sanity check for schema fails.
        """
        errors = []
        schema = self.load_schema()
        validator = validator_for(schema)

        try:
            validator.check_schema(schema)
        except SchemaError as schema_err:
            msg = "Sanity check failed for in-built schema. This is likely a bug.\n"
            raise CreatorError(
                msg,
            ) from schema_err

        validation_errors = sorted(
            validator(schema).iter_errors(self.data),
            key=lambda e: e.path,
        )

        for err in validation_errors:
            err_msg = "* " + str(err.message) + " at " + str(err.instance) + "\n"
            errors.append(err_msg)

        return errors

    def load_schema(self: SchemaValidator) -> dict:
        """Attempt to load the schema from schemas directory.

        :returns: A schema loaded as json.
        :raises CreatorError: if the JSON schema cannot be loaded.
        """
        try:
            schema: dict = loads(get_file_contents("schemas", self.criteria))
        except (
            FileNotFoundError,
            TypeError,
            JSONDecodeError,
            ModuleNotFoundError,
        ) as err:
            msg = "Unable to load jsonschema for validation with error(s):\n"
            raise CreatorError(
                msg,
            ) from err

        return schema



"""Definitions for ansible-creator init action."""

from __future__ import annotations

import os
import shutil

from typing import TYPE_CHECKING

from ansible_creator.exceptions import CreatorError
from ansible_creator.templar import Templar
from ansible_creator.utils import copy_container


if TYPE_CHECKING:
    from ansible_creator.config import Config
    from ansible_creator.output import Output


class Init:
    """Class representing ansible-creator create subcommand."""

    def __init__(
        self: Init,
        config: Config,
        output: Output,
    ) -> None:
        """Initialize the init action.

        :param config: App configuration object.
        :param output: Output class object.
        """
        self._namespace: str = config.namespace
        self._collection_name: str = config.collection_name
        self._init_path: str = config.init_path
        self._force = config.force
        self._creator_version = config.creator_version
        self._templar = Templar()
        self.output: Output = output

    def run(self: Init) -> None:
        """Start scaffolding collection skeleton.

        :raises CreatorError: if computed collection path is an existing directory or file.
        """
        col_path = os.path.join(self._init_path, self._namespace, self._collection_name)

        self.output.debug(msg=f"final collection path set to {col_path}")

        # check if init_path already exists
        if os.path.exists(col_path):
            if os.path.isfile(col_path):
                msg = f"the path {col_path} already exists, but is a file - aborting"
                raise CreatorError(
                    msg,
                )

            if not self._force:
                msg = (
                    f"The directory {col_path} already exists.\n"
                    f"You can use --force to re-initialize this directory."
                    f"\nHowever it will delete ALL existing contents in it."
                )
                raise CreatorError(msg)

            # user requested --force, re-initializing existing directory
            self.output.warning(f"re-initializing existing directory {col_path}")
            for root, dirs, files in os.walk(col_path, topdown=True):
                for old_dir in dirs:
                    path = os.path.join(root, old_dir)
                    self.output.debug(f"removing tree {old_dir}")
                    shutil.rmtree(path)
                for old_file in files:
                    path = os.path.join(root, old_file)
                    self.output.debug(f"removing file {old_file}")
                    os.unlink(path)

        # if init_path does not exist, create it
        if not os.path.exists(col_path):
            self.output.debug(msg=f"creating new directory at {col_path}")
            os.makedirs(col_path)

        # copy new_collection container to destination, templating files when found
        self.output.debug(msg="started copying collection skeleton to destination")
        copy_container(
            source="new_collection",
            dest=col_path,
            templar=self._templar,
            template_data={
                "namespace": self._namespace,
                "collection_name": self._collection_name,
                "creator_version": self._creator_version,
            },
            output=self.output,
        )

        self.output.note(
            f"collection {self._namespace}.{self._collection_name} created at {self._init_path}",
        )



"""Application configuration class for ansible-creator."""

from __future__ import annotations

from dataclasses import dataclass, field

from ansible_creator.utils import expand_path


@dataclass(frozen=True)
class Config:
    """The application configuration for ansible-creator."""

    creator_version: str
    json: bool
    log_append: bool
    log_file: str
    log_level: str
    no_ansi: bool
    subcommand: str
    verbose: int

    collection: str = ""
    force: bool = False
    init_path: str = "./"

    # TO-DO: Add instance variables for other 'create' and 'sample'

    collection_name: str = field(init=False)
    namespace: str = field(init=False)

    def __post_init__(self: Config) -> None:
        """Post process config values."""
        if self.collection:
            fqcn = self.collection.split(".", maxsplit=1)
            object.__setattr__(self, "namespace", fqcn[0])
            object.__setattr__(self, "collection_name", fqcn[-1])

        object.__setattr__(self, "init_path", expand_path(self.init_path))



"""A Jinja2 template engine."""
from __future__ import annotations


try:
    from jinja2 import Environment, StrictUndefined

    HAS_JINJA2 = True
except ImportError:
    HAS_JINJA2 = False

from ansible_creator.utils import get_file_contents


class Templar:
    """Class representing a Jinja2 template engine."""

    def __init__(self: Templar) -> None:
        """Instantiate the template engine.

        :raises ImportError: When jinja2 is not installed.
        """
        if not HAS_JINJA2:
            msg = (
                "jinja2 is required but does not appear to be installed."
                "It can be installed using `pip install jinja2`"
            )
            raise ImportError(
                msg,
            )
        self.env: Environment = Environment(  # noqa: S701
            undefined=StrictUndefined,
            keep_trailing_newline=True,
        )

    def render(self: Templar, template_name: str, data: dict) -> str:
        """Load template from a file and render with provided data.

        :param template_name: Name of the template to load.
        :param data: Data to render template with.

        :returns: Templated content.
        """
        template_content = get_file_contents(
            directory="templates",
            filename=template_name,
        )
        return self.render_from_content(template=template_content, data=data)

    def render_from_content(self: Templar, template: str, data: dict) -> str:
        """Render a template with provided data.

        :param template: The template to load and render.
        :param data: Data to render template with.

        :returns: Templated content.
        """
        return self.env.from_string(template).render(data)



"""Definition of constants for this package."""

MessageColors = {
    "HEADER": "\033[94m",
    "OKGREEN": "\033[92m",
    "WARNING": "\033[93m",
    "FAILURE": "\033[1;31m",
    "OK": "\033[95m",
    "ENDC": "\033[0m",
}

OPTION_CONDITIONALS = (
    "mutually_exclusive",
    "required_one_of",
    "required_together",
    "required_by",
    "required_if",
)

OPTION_METADATA = (
    "type",
    "choices",
    "default",
    "required",
    "aliases",
    "elements",
    "fallback",
    "no_log",
    "apply_defaults",
    "deprecated_aliases",
    "removed_in_version",
)

VALID_ANSIBLEMODULE_ARGS = (
    "bypass_checks",
    "no_log",
    "add_file_common_args",
    "supports_check_mode",
    *OPTION_CONDITIONALS,
)



"""The ansible-creator Cli."""

from __future__ import annotations

import argparse
import os
import sys

from importlib import import_module
from pathlib import Path
from typing import TYPE_CHECKING

from ansible_creator.config import Config
from ansible_creator.exceptions import CreatorError
from ansible_creator.output import Output
from ansible_creator.utils import TermFeatures


try:
    from ._version import version as __version__
except ImportError:
    __version__ = "source"

if TYPE_CHECKING:
    from argparse import Namespace


class Cli:
    """Class representing the ansible-creator Cli."""

    def __init__(self: Cli) -> None:
        """Initialize the Cli and parse Cli args."""
        self.args: Namespace = self.parse_args()
        self.output: Output
        self.term_features: TermFeatures

    def init_output(self: Cli) -> None:
        """Initialize the output object."""
        no_ansi = self.args.no_ansi
        if not sys.stdout.isatty():
            self.term_features = TermFeatures(color=False, links=False)
        else:
            self.term_features = TermFeatures(
                color=False if os.environ.get("NO_COLOR") else not no_ansi,
                links=not no_ansi,
            )

        self.output = Output(
            log_append=self.args.log_append,
            log_file=self.args.log_file,
            log_level=self.args.log_level,
            term_features=self.term_features,
            verbosity=self.args.verbose,
            display="json" if self.args.json else "text",
        )

    def parse_args(self: Cli) -> argparse.Namespace:
        """Start parsing args passed from Cli.

        :returns: A dictionary of Cli args.
        """
        parent_parser = argparse.ArgumentParser(add_help=False)

        parent_parser.add_argument(
            "--na",
            "--no-ansi",
            action="store_true",
            default=False,
            dest="no_ansi",
            help="Disable the use of ANSI codes for terminal color.",
        )

        parent_parser.add_argument(
            "--lf",
            "--log-file <file>",
            dest="log_file",
            default=str(Path.cwd() / "ansible-creator.log"),
            help="Log file to write to.",
        )

        parent_parser.add_argument(
            "--ll",
            "--log-level <level>",
            dest="log_level",
            default="notset",
            choices=["notset", "debug", "info", "warning", "error", "critical"],
            help="Log level for file output.",
        )

        parent_parser.add_argument(
            "--la",
            "--log-append <bool>",
            dest="log_append",
            choices=["true", "false"],
            default="true",
            help="Append to log file.",
        )

        parent_parser.add_argument(
            "--json",
            dest="json",
            action="store_true",
            default=False,
            help="Output messages as JSON",
        )

        parent_parser.add_argument(
            "-v",
            dest="verbose",
            action="count",
            default=0,
            help="Give more Cli output. Option is additive, and can be used up to 3 times.",
        )

        parser = argparse.ArgumentParser(
            description=(
                "Tool to scaffold Ansible Content. Get started by looking at the help text."
            ),
        )

        parser.add_argument(
            "--version",
            action="version",
            version=__version__,
            help="Print ansible-creator version and exit.",
        )

        subparsers = parser.add_subparsers(
            help="The subcommand to invoke.",
            title="Commands",
            dest="subcommand",
        )
        subparsers.required = True

        # 'init' command parser

        init_command_parser = subparsers.add_parser(
            "init",
            help="Initialize an Ansible Collection.",
            description=("Creates the skeleton framework of an Ansible collection."),
            parents=[parent_parser],
        )

        init_command_parser.add_argument(
            "collection",
            help="The collection name in the format ``<namespace>.<collection>``.",
        )

        init_command_parser.add_argument(
            "--init-path",
            default="./",
            help="The path in which the skeleton collection will be created. The default is the "
            "current working directory.",
        )

        init_command_parser.add_argument(
            "--force",
            default=False,
            action="store_true",
            help="Force re-initialize the specified directory as an Ansible collection.",
        )

        return parser.parse_args()

    def run(self: Cli) -> None:
        """Dispatch work to correct subcommand class."""
        self.output.debug(msg=f"parsed args {self.args!s}")
        subcommand = self.args.subcommand
        subcommand_module = f"ansible_creator.subcommands.{subcommand}"
        subcommand_cls = f"{subcommand}".capitalize()
        args = vars(self.args)
        args.update({"creator_version": __version__})

        try:
            self.output.debug(msg=f"starting requested action '{subcommand}'")
            subcommand = getattr(import_module(subcommand_module), subcommand_cls)
            self.output.debug(f"found action class {subcommand}")
            subcommand(config=Config(**args), output=self.output).run()
        except CreatorError as exc:
            self.output.error(str(exc))
            sys.exit(1)

        self.output.debug(msg="exiting ansible-creator")


def main() -> None:
    """Entry point for ansible-creator Cli."""
    cli = Cli()
    cli.init_output()
    cli.run()


if __name__ == "__main__":
    main()
